import arcpy, ArcSAGA

class Toolbox(object):
	def __init__(self):
		self.label = "ODBC/OTL"
		self.alias = ""
		self.tools = [tool_0, tool_1, tool_2, tool_3, tool_4, tool_5, tool_7, tool_8, tool_9, tool_10, tool_11]

class tool_0(object):
	def __init__(self):
		self.label = "Connect to ODBC Source"
		self.description = "<p>Connects to an ODBC source.<p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B_hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="Server", name="SERVER", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["		param.value = "<no choice available>"
		params  = [param]
		param = arcpy.Parameter(displayName="User", name="USERNAME", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Password", name="PASSWORD", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('db_odbc', '0')
		Tool.Set_Option('SERVER', parameters[0].valueAsText)
		Tool.Set_Option('USERNAME', parameters[1].valueAsText)
		Tool.Set_Option('PASSWORD', parameters[2].valueAsText)
		Tool.Run()
		return


class tool_1(object):
	def __init__(self):
		self.label = "Disconnect from ODBC Source"
		self.description = "<p>Disconnects an ODBC source connection.<p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B_hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="DSN", name="ODBC_DSN", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params  = [param]
		param = arcpy.Parameter(displayName="User", name="ODBC_USR", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Password", name="ODBC_PWD", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Transactions", name="TRANSACT", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["rollback", "commit"]
		param.value = "commit"
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('db_odbc', '1')
		Tool.Set_Option('ODBC_DSN', parameters[0].valueAsText)
		Tool.Set_Option('ODBC_USR', parameters[1].valueAsText)
		Tool.Set_Option('ODBC_PWD', parameters[2].valueAsText)
		Tool.Set_Option('TRANSACT', parameters[3].valueAsText)
		Tool.Run()
		return


class tool_2(object):
	def __init__(self):
		self.label = "Commit/Rollback Transaction"
		self.description = "<p>Execute a commit or rollback on open transactions with ODBC source.<p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B_hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="DSN", name="ODBC_DSN", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params  = [param]
		param = arcpy.Parameter(displayName="User", name="ODBC_USR", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Password", name="ODBC_PWD", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Server", name="SERVERS", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["		param.value = "<no choice available>"
		params += [param]
		param = arcpy.Parameter(displayName="Transactions", name="TRANSACT", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["rollback", "commit"]
		param.value = "commit"
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('db_odbc', '2')
		Tool.Set_Option('ODBC_DSN', parameters[0].valueAsText)
		Tool.Set_Option('ODBC_USR', parameters[1].valueAsText)
		Tool.Set_Option('ODBC_PWD', parameters[2].valueAsText)
		Tool.Set_Option('SERVERS', parameters[3].valueAsText)
		Tool.Set_Option('TRANSACT', parameters[4].valueAsText)
		Tool.Run()
		return


class tool_3(object):
	def __init__(self):
		self.label = "Execute SQL"
		self.description = "<p>Execute SQL commands on a connected ODBC source. Separate different commands with a semicolon (';'). <p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B_hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="DSN", name="ODBC_DSN", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params  = [param]
		param = arcpy.Parameter(displayName="User", name="ODBC_USR", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Password", name="ODBC_PWD", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="SQL Statment", name="SQL", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = "CREATE TABLE myTable1 (Col1 VARCHAR(255) PRIMARY KEY, Col2 INTEGER);\nINSERT INTO myTable1 (Col1, Col2) VALUES('First Value', 1);\nDROP TABLE myTable1;\n"
		params += [param]
		param = arcpy.Parameter(displayName="Commit", name="COMMIT", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = True
		params += [param]
		param = arcpy.Parameter(displayName="Stop on Error", name="STOP", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('db_odbc', '3')
		Tool.Set_Option('ODBC_DSN', parameters[0].valueAsText)
		Tool.Set_Option('ODBC_USR', parameters[1].valueAsText)
		Tool.Set_Option('ODBC_PWD', parameters[2].valueAsText)
		Tool.Set_Option('SQL', parameters[3].valueAsText)
		Tool.Set_Option('COMMIT', parameters[4].valueAsText)
		Tool.Set_Option('STOP', parameters[5].valueAsText)
		Tool.Run()
		return


class tool_4(object):
	def __init__(self):
		self.label = "List Table Fields"
		self.description = "<p>Loads table information from ODBC data source.<p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B_hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="DSN", name="ODBC_DSN", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params  = [param]
		param = arcpy.Parameter(displayName="User", name="ODBC_USR", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Password", name="ODBC_PWD", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Field Description", name="TABLE", direction="Output", parameterType="Required", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Tables", name="TABLES", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["		param.value = "<no choice available>"
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('db_odbc', '4')
		Tool.Set_Option('ODBC_DSN', parameters[0].valueAsText)
		Tool.Set_Option('ODBC_USR', parameters[1].valueAsText)
		Tool.Set_Option('ODBC_PWD', parameters[2].valueAsText)
		Tool.Set_Output('TABLE', parameters[3].valueAsText, 'table')
		Tool.Set_Option('TABLES', parameters[4].valueAsText)
		Tool.Run()
		return


class tool_5(object):
	def __init__(self):
		self.label = "Import Table"
		self.description = "<p>Imports a table from a database via ODBC.<p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B_hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="DSN", name="ODBC_DSN", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params  = [param]
		param = arcpy.Parameter(displayName="User", name="ODBC_USR", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Password", name="ODBC_PWD", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Table", name="TABLE", direction="Output", parameterType="Required", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Tables", name="TABLES", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["		param.value = "<no choice available>"
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('db_odbc', '5')
		Tool.Set_Option('ODBC_DSN', parameters[0].valueAsText)
		Tool.Set_Option('ODBC_USR', parameters[1].valueAsText)
		Tool.Set_Option('ODBC_PWD', parameters[2].valueAsText)
		Tool.Set_Output('TABLE', parameters[3].valueAsText, 'table')
		Tool.Set_Option('TABLES', parameters[4].valueAsText)
		Tool.Run()
		return


class tool_7(object):
	def __init__(self):
		self.label = "Drop Table"
		self.description = "<p>Deletes a table from a database via ODBC.<p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B_hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="DSN", name="ODBC_DSN", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params  = [param]
		param = arcpy.Parameter(displayName="User", name="ODBC_USR", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Password", name="ODBC_PWD", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Tables", name="TABLES", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["		param.value = "<no choice available>"
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('db_odbc', '7')
		Tool.Set_Option('ODBC_DSN', parameters[0].valueAsText)
		Tool.Set_Option('ODBC_USR', parameters[1].valueAsText)
		Tool.Set_Option('ODBC_PWD', parameters[2].valueAsText)
		Tool.Set_Option('TABLES', parameters[3].valueAsText)
		Tool.Run()
		return


class tool_8(object):
	def __init__(self):
		self.label = "Import Table from SQL Query"
		self.description = "<p>Import a SQL table from a database via ODBC.<p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B_hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="DSN", name="ODBC_DSN", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params  = [param]
		param = arcpy.Parameter(displayName="User", name="ODBC_USR", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Password", name="ODBC_PWD", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Table from SQL Query", name="TABLE", direction="Output", parameterType="Required", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Tables", name="TABLES", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Fields", name="FIELDS", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = "*"
		params += [param]
		param = arcpy.Parameter(displayName="Where", name="WHERE", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Group by", name="GROUP", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Having", name="HAVING", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Order by", name="ORDER", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Distinct", name="DISTINCT", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('db_odbc', '8')
		Tool.Set_Option('ODBC_DSN', parameters[0].valueAsText)
		Tool.Set_Option('ODBC_USR', parameters[1].valueAsText)
		Tool.Set_Option('ODBC_PWD', parameters[2].valueAsText)
		Tool.Set_Output('TABLE', parameters[3].valueAsText, 'table')
		Tool.Set_Option('TABLES', parameters[4].valueAsText)
		Tool.Set_Option('FIELDS', parameters[5].valueAsText)
		Tool.Set_Option('WHERE', parameters[6].valueAsText)
		Tool.Set_Option('GROUP', parameters[7].valueAsText)
		Tool.Set_Option('HAVING', parameters[8].valueAsText)
		Tool.Set_Option('ORDER', parameters[9].valueAsText)
		Tool.Set_Option('DISTINCT', parameters[10].valueAsText)
		Tool.Run()
		return


class tool_9(object):
	def __init__(self):
		self.label = "List ODBC Servers"
		self.description = "<p>Lists all ODBC sources.<p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B_hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="Server", name="SERVERS", direction="Output", parameterType="Required", datatype="GPTableView")
		params  = [param]
		param = arcpy.Parameter(displayName="Only List Connected Servers", name="CONNECTED", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('db_odbc', '9')
		Tool.Set_Output('SERVERS', parameters[0].valueAsText, 'table')
		Tool.Set_Option('CONNECTED', parameters[1].valueAsText)
		Tool.Run()
		return


class tool_10(object):
	def __init__(self):
		self.label = "List Tables"
		self.description = "<p>Lists all tables of an ODBC data source.<p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B_hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="DSN", name="ODBC_DSN", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params  = [param]
		param = arcpy.Parameter(displayName="User", name="ODBC_USR", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Password", name="ODBC_PWD", direction="Input", parameterType="Optional", datatype="GPString")
		param.value = ""
		params += [param]
		param = arcpy.Parameter(displayName="Tables", name="TABLES", direction="Output", parameterType="Required", datatype="GPTableView")
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('db_odbc', '10')
		Tool.Set_Option('ODBC_DSN', parameters[0].valueAsText)
		Tool.Set_Option('ODBC_USR', parameters[1].valueAsText)
		Tool.Set_Option('ODBC_PWD', parameters[2].valueAsText)
		Tool.Set_Output('TABLES', parameters[3].valueAsText, 'table')
		Tool.Run()
		return


class tool_11(object):
	def __init__(self):
		self.label = "Disconnect All"
		self.description = "<p>Disconnects all connected ODBC sources.<p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B_hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="Transactions", name="TRANSACT", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["rollback", "commit"]
		param.value = "commit"
		params  = [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('db_odbc', '11')
		Tool.Set_Option('TRANSACT', parameters[0].valueAsText)
		Tool.Run()
		return
